"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ZEServices = void 0;
require("isomorphic-fetch");
class ZEServices {
    //#endregion
    constructor() {
        this.jwt = null;
        this.token = null;
        this.gigyaProd = null;
        this.wiredProd = null;
        this.country = "DE";
    }
    //#region Parse JWT Function
    atob(a) {
        return Buffer.from(a, 'base64').toString('binary');
    }
    parseJwt(token) {
        var base64Url = token.split('.')[1];
        var base64 = base64Url.replace(/-/g, '+').replace(/_/g, '/');
        var jsonPayload = decodeURIComponent(this.atob(base64).split('').map(function (c) {
            return '%' + ('00' + c.charCodeAt(0).toString(16)).slice(-2);
        }).join(''));
        return JSON.parse(jsonPayload);
    }
    /**
     * Login into the renault service and fetch on success the jwt for all further API Calls.
     * @param loginId Username for the renault service.
     * @param password Password for the renault service.
     */
    login(loginId, password) {
        return __awaiter(this, void 0, void 0, function* () {
            let configURL = 'https://renault-wrd-prod-1-euw1-myrapp-one.s3-eu-west-1.amazonaws.com/configuration/android/config_en_GB.json';
            // add default config if the renault server is not available
            let config = {
                servers: {
                    gigyaProd: {
                        target: 'https://accounts.eu1.gigya.com',
                        apikey: '3_7PLksOyBRkHv126x5WhHb-5pqC1qFR8pQjxSeLB6nhAnPERTUlwnYoznHSxwX668'
                    },
                    wiredProd: {
                        target: 'https://api-wired-prod-1-euw1.wrd-aws.com',
                        apikey: 'VAX7XYKGfa92yMvXculCkEFyfZbuM7Ss'
                    }
                }
            };
            try {
                config = yield (yield fetch(configURL, { method: "GET" })).json();
            }
            catch (_a) { }
            this.gigyaProd = config.servers.gigyaProd;
            this.wiredProd = config.servers.wiredProd;
            // manual override kameron API key searching for new fetchable source
            this.wiredProd.apikey = "VAX7XYKGfa92yMvXculCkEFyfZbuM7Ss";
            let oauth = yield (yield fetch(this.gigyaProd.target
                + "/accounts.login"
                + "?apikey=" + this.gigyaProd.apikey
                + "&loginID=" + encodeURIComponent(loginId)
                + "&password=" + encodeURIComponent(password), { method: "POST" })).json();
            this.token = oauth.sessionInfo.cookieValue;
            let json = yield (yield fetch(this.gigyaProd.target
                + "/accounts.getJWT"
                + "?apikey=" + this.gigyaProd.apikey
                + "&fields=data.personId,data.gigyaDataCenter"
                + "&expiration=3600" // TTL in seconds default 300
                + "&oauth_token=" + this.token
                + "&login_token=" + this.token, { method: "POST" })).json();
            this.jwt = json.id_token;
            return true;
        });
    }
    getJSON(PATH, country) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.jwt == null)
                return new Promise((resolve) => { resolve(null); });
            if (!country)
                country = this.country;
            var res = yield fetch(this.wiredProd.target + PATH + '?country=' + country, {
                method: "GET",
                headers: {
                    "apikey": this.wiredProd.apikey,
                    "x-gigya-id_token": this.jwt
                }
            });
            return res.json();
        });
    }
    postJSON(data, PATH, country) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.jwt == null)
                return new Promise((resolve) => { resolve(null); });
            if (!country)
                country = this.country;
            var res = yield fetch(this.wiredProd.target + PATH + '?country=' + country, {
                method: "POST",
                body: JSON.stringify(data),
                headers: {
                    "apikey": this.wiredProd.apikey,
                    "x-gigya-id_token": this.jwt,
                    "Content-Type": "application/vnd.api+json"
                }
            });
            return res.json();
        });
    }
    createPath(accountId, vin, version = 1) {
        return "/commerce/v1/accounts/" + accountId + "/kamereon/kca/car-adapter/v" + version + "/cars/" + vin;
    }
    createPathSpring(accountId, vin) {
        return "/commerce/v1/accounts/" + accountId + "/kamereon/kcm/v1/vehicles/" + vin;
    }
    /**
     * Fetch all accounts
     * @param personId optional personId, if not defined taken from the jwt
     * @param country optional country
     */
    accounts(personId, country) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!personId)
                personId = this.parseJwt(this.jwt)["data.personId"];
            return this.getJSON("/commerce/v1/persons/" + personId, country);
        });
    }
    /**
     * Fetch all vehicles that in this account.
     * @param accountId The accountId.
     * @param country optional country
     */
    vehicles(accountId, country) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.getJSON("/commerce/v1/accounts/" + accountId + "/vehicles", country);
        });
    }
    getAttribute(attribute, accountId, vin, country) {
        return __awaiter(this, void 0, void 0, function* () {
            let version = 1;
            switch (attribute) {
                case 'cockpit':
                case 'battery-status':
                    version = 2;
                    break;
            }
            return this.getJSON(this.createPath(accountId, vin, version) + "/" + attribute, country);
        });
    }
    /**
     * Fetch the current location of the requested vehicle in the requested account.
     * @param accountId The accountId.
     * @param vin The vehicle identifier.
     * @param country optional country
     */
    location(accountId, vin, country) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.getAttribute("location", accountId, vin, country);
        });
    }
    /**
     * Fetch the current cockpit status of the requested vehicle in the requested account.
     * @param accountId The accountId.
     * @param vin The vehicle identifier.
     * @param country optional country
     */
    cockpit(accountId, vin, country) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.getAttribute("cockpit", accountId, vin, country);
        });
    }
    /**
     * Fetch the current battery status of the requested vehicle in the requested account.
     * @param accountId The accountId.
     * @param vin The vehicle identifier.
     * @param country optional country
     */
    batteryStatus(accountId, vin, country) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.getAttribute("battery-status", accountId, vin, country);
        });
    }
    /**
     * Fetch the current charge mode of the requested vehicle in the requested account.
     * @param accountId The accountId.
     * @param vin The vehicle identifier.
     * @param country optional country
     */
    chargeMode(accountId, vin, country) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.getAttribute("charge-mode", accountId, vin, country);
        });
    }
    /**
     * Fetch the current ac schedule of the requested vehicle in the requested account.
     * @param accountId The accountId.
     * @param vin The vehicle identifier.
     * @param country optional country
     */
    hvacSchedule(accountId, vin, country) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.getAttribute("hvac-schedule", accountId, vin, country);
        });
    }
    /**
     * Fetch the current charge schedule of the requested vehicle in the requested account.
     * @param accountId The accountId.
     * @param vin The vehicle identifier.
     * @param country optional country
     */
    chargeSchedule(accountId, vin, country) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.getAttribute("charge-schedule", accountId, vin, country);
        });
    }
    /**
     * Set the charge mode of the defined vehicle in the defined account.
     * @param mode The charge mode can be "always_charging"/"always" or "schedule_mode"/"schedule".
     * @param accountId The accountId.
     * @param vin The vehicle identifier.
     * @param country optional country
     */
    setChargeMode(mode, accountId, vin, country) {
        return __awaiter(this, void 0, void 0, function* () {
            if (mode == "always")
                mode = "always_charging";
            if (mode == "schedule")
                mode = "schedule_mode";
            let data = {
                "data": {
                    "type": "ChargeMode",
                    "attributes": {
                        "action": mode
                    }
                }
            };
            return this.postJSON(data, this.createPath(accountId, vin) + "/actions/charge-mode", country);
        });
    }
    /**
     * Set the charge state of the defined vehicle in the defined account.
     * @param charging With true the car tries to start charging, with false it stops the charging.
     * @param accountId The accountId.
     * @param vin The vehicle identifier.
     * @param country optional country
     */
    setChargeState(charging, accountId, vin, country) {
        return __awaiter(this, void 0, void 0, function* () {
            //  "startDateTime" with format "YYYY-MM-DDThh:mm:ssZ"
            let data = {
                "data": {
                    "type": "ChargingStart",
                    "attributes": {
                        "action": charging ? "start" : "stop"
                    }
                }
            };
            return this.postJSON(data, this.createPath(accountId, vin) + "/actions/charging-start", country);
        });
    }
    /**
     * Pause and resume Charging for Dacia Spring.
     * @param pause With true the car tries to pause charging, with false it resumes the charging.
     * @param accountId The accountId.
     * @param vin The vehicle identifier.
     * @param country optional country
     */
    setChargePause(pause, accountId, vin, country) {
        return __awaiter(this, void 0, void 0, function* () {
            let data = {
                "data": {
                    "type": "ChargePauseResume",
                    "attributes": {
                        "action": pause ? "pause" : "resume"
                    }
                }
            };
            return this.postJSON(data, this.createPathSpring(accountId, vin) + "/charge/pause-resume", country);
        });
    }
    /**
     * Set the AC state. Starts the AC with the targettemperature or stops with targettemperature = null.
     * @param temperature The targettemperature for the ac.
     * @param accountId The accountId.
     * @param vin The vehicle identifier.
     * @param country optional country
     */
    setHVACState(targetTemperature, accountId, vin, country) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof targetTemperature === "boolean") {
                if (targetTemperature)
                    targetTemperature = 21;
                else
                    targetTemperature = undefined;
            }
            let ac = !(targetTemperature === null);
            if (!targetTemperature && targetTemperature < 0 && targetTemperature > 30)
                targetTemperature = 21;
            //  "startDateTime" with format "YYYY-MM-DDThh:mm:ssZ"
            let data = {
                "data": {
                    "type": "HvacStart",
                    "attributes": {
                        "action": ac ? "start" : "stop",
                        "targetTemperature": targetTemperature
                    }
                }
            };
            if (!ac)
                delete data.data.attributes.targetTemperature;
            return this.postJSON(data, this.createPath(accountId, vin) + "/actions/hvac-start", country);
        });
    }
    /**
     * Set the charge schedule.
     * @param shedule The schedule, in the format { data: { attributes: { calendar: ...} }}
     * @param accountId The accountId.
     * @param vin The vehicle identifier.
     * @param country optional country
     */
    setChargeSchedule(shedule, accountId, vin, country) {
        return __awaiter(this, void 0, void 0, function* () {
            shedule.data.type = "ChargeSchedule";
            return this.postJSON(shedule, this.createPath(accountId, vin, 2) + "/actions/charge-schedule ", country);
        });
    }
    /**
     * Set the HACV schedule.
     * @param shedule The schedule.
     * @param accountId The accountId.
     * @param vin The vehicle identifier.
     * @param country optional country
     */
    setHVACSchedule(shedule, accountId, vin, country) {
        return __awaiter(this, void 0, void 0, function* () {
            shedule.data.type = "HvacSchedule";
            return this.postJSON(shedule, this.createPath(accountId, vin) + "/actions/hvac-schedule ", country);
        });
    }
}
exports.ZEServices = ZEServices;
