"use strict";
const ZEServices_1 = require("./ZEServices");
module.exports = function (RED) {
    function RenaultZENode(config) {
        var node = this;
        RED.nodes.createNode(node, config);
        let ZE = new ZEServices_1.ZEServices();
        node.on('input', (msg) => {
            var _a, _b;
            let accountId = null;
            let vin = null;
            let topicList = [];
            if (!msg.topic) {
                topicList.push('location');
                topicList.push('cockpit');
                topicList.push('battery-status');
            }
            else
                topicList.push(msg.topic.toLowerCase());
            let username = (_a = msg.username) !== null && _a !== void 0 ? _a : node.credentials.username;
            let password = (_b = msg.password) !== null && _b !== void 0 ? _b : node.credentials.password;
            ZE.login(username, password)
                .then(() => {
                return ZE.accounts();
            })
                .then((accounts) => {
                let result = [];
                for (let item of accounts.accounts) {
                    result.push(ZE.vehicles(item.accountId));
                }
                return Promise.all(result);
            })
                .then((accountvehicles) => {
                for (let account of accountvehicles) {
                    for (let vehicles of account.vehicleLinks) {
                        // if a vin is defined only this will be fetched
                        if (!(!(msg === null || msg === void 0 ? void 0 : msg.vin)) && vehicles.vin != msg.vin) {
                            break;
                        }
                        for (let topic of topicList) {
                            switch (topic) {
                                case "set-charge-mode":
                                    ZE.setChargeMode(msg.payload, account.accountId, vehicles.vin, account.country)
                                        .then((result) => node.send(Object.assign(Object.assign({}, msg), { topic: topic, payload: result })));
                                    break;
                                case "set-charge-state":
                                    ZE.setChargeState(msg.payload, account.accountId, vehicles.vin, account.country)
                                        .then((result) => node.send(Object.assign(Object.assign({}, msg), { topic: topic, payload: result })));
                                    break;
                                case "set-charge-pause":
                                    ZE.setChargePause(msg.payload, account.accountId, vehicles.vin, account.country)
                                        .then((result) => node.send(Object.assign(Object.assign({}, msg), { topic: topic, payload: result })));
                                    break;
                                case "set-hvac-state":
                                    ZE.setHVACState(msg.payload, account.accountId, vehicles.vin, account.country)
                                        .then((result) => node.send(Object.assign(Object.assign({}, msg), { topic: topic, payload: result })));
                                    break;
                                case "set-charge-schedule":
                                    ZE.setChargeSchedule(msg.payload, account.accountId, vehicles.vin, account.country)
                                        .then((result) => node.send(Object.assign(Object.assign({}, msg), { topic: topic, payload: result })));
                                    break;
                                case "set-hvac-schedule":
                                    ZE.setHVACSchedule(msg.payload, account.accountId, vehicles.vin, account.country)
                                        .then((result) => node.send(Object.assign(Object.assign({}, msg), { topic: topic, payload: result })));
                                    break;
                                default:
                                    ZE.getAttribute(topic, account.accountId, vehicles.vin, account.country)
                                        .then((result) => node.send(Object.assign(Object.assign({}, msg), { topic: topic, payload: result })));
                                    break;
                            }
                        }
                    }
                }
            })
                .catch((err) => {
                node.error(err);
            });
        });
    }
    RED.nodes.registerType("renault-ze", RenaultZENode, {
        credentials: {
            username: { type: "text" },
            password: { type: "password" }
        }
    });
};
